/*
---------------------------------------------------------------------------
 $Revision: 1.1.2.6.10.1 $                                                                 
 $Date: 2017/09/13 17:26:52 $  

 NAME: sea_dq_provider_hierarchy

 TYPE: macro

 DESCRIPTION:
   Perform the data quality checks on the SEA_PROVIDER_HIERARCHY table.
  
 SAS VERSION:
 9.4
   
 PARAMETERS: 
 
 REQUIREMENT: 

 HISTORY:
 userid yyyymmdd defectid desc    
 sascos 20150401          Initial
-----------------------------------------------------------------------------
 Copyright (c) 2014, SAS Institute Inc., Cary, NC, USA, All Rights Reserved 
------------------------------------------------------------------------------
*/

%macro sea_dq_provider_hierarchy (indsn=,outdsn=);

   /* 
   * Create global macro variables
   */
   %global sea_provider_hierarchy_duprecs 
           sea_provider_hierarchy_errors;
   %let sea_provider_hierarchy_duprecs=0;
   %let sea_provider_hierarchy_errors=0;
   %let retcode=0;
   %let inlib=%scan(&indsn,1,.);

   /* get required variables to test for missing values */
   %let rq=0;
   data _null_;
      set required_fields end=eof;
      call symput(compress('rq_'||put(_n_,best.)),strip(stmt));
      call symput(compress('rqf_'||put(_n_,best.)),strip(col));
      if eof then call symput('rq',put(_n_,best.));
   run;

   /* Error if missing values */
   %if &rq=0 %then %do;
      %let retcode=1;
      %let errmsg_enroll=ERROR: Required fields missing;
      %GOTO dq_provhier_end;
   %end;  

   /* Check for indexes */
   %let index=0;
   proc sql noprint;
      select '1' as index into :index
      from dictionary.indexes
      where libname=upcase("&inlib")
         and memname=upcase("%scan(&indsn,2,.)")
         and upcase(indxname)='PROVIDER_HIERARCHY';
   quit;

   /* Add index to sea_provider_hier */
   %if &index=0 %then %do;
      proc datasets lib=&inlib nolist nowarn;
         modify %scan(&indsn,2,.);
         index create provider_hier=(provider_hierarchy_nm provider_id); 
      run;
      quit;
   %end;

   /* Dedupe */
   proc sort data=&provider (keep=provider_id) out=work.for_fmt nodupkey;
      by provider_id;
   run;

   /* Create a format with the provider_ids */
   data work.for_fmt (drop=provider_id);
      attrib start length=$32.;
      retain fmtname '$prvlst';
	  retain type 'C';
      set for_fmt;
      start=trim(left(provider_id)); 
      label='Found';
   run;

   proc format cntlin=for_fmt;
   run;

   /* Read the data in and check the conditions */
   data &outdsn (keep=provider_id provider_hierarchy_nm 
                      field dq_check reason type);

      attrib field     length=$32  label='Field with DQ Issue'
             dq_check  length=$100  label='Data Quality Check'
             reason    length=$100 label='Field Value with DQ Issue'
             type      length=$1;

      set &indsn end=eof;
      by provider_hierarchy_nm provider_id;
      retain errors duprecs 0;

      type='E';

      /* required fields - missing values */
      %do rz=1 %to &rq;
         if &&rq_&rz then do;
            errors=errors+1;
            field="&&rqf_&rz";
            dq_check="Field not populated.";
            reason="No data";
            output &outdsn; 
         end;
      %end;

      /* Other checks */
      if trim(left(provider_id)) ne '' then do;
         if put(provider_id,$prvlst.) ne 'Found' then do;
            errors=errors+1;
            field="PROVIDER_ID";
            dq_check="Invalid Provider_ID value in Provider_ID column";
            reason=provider_id;
            output &outdsn; 
         end;
      end;

      /* If the hierarchy fields contain a value, it should be valid */
      %do x=1 %to 5;
         if trim(left(hl_&x)) ne '' then 
         do; 
            if put(hl_&x,$prvlst.) ne 'Found' then 
            do;
               errors=errors+1;
               field="HL_&X";
               dq_check="Invalid Provider_ID value in HL_&X column.";
               reason=hl_&x;
               output &outdsn;
            end;
         end;
      %end;

      if eof then do;
         call symput('sea_provider_hierarchy_errors',put(errors,best.));
         call symput('sea_provider_hierarchy_duprecs',put(duprecs,best.));
      end;
   run;

   %if &sea_provider_hierarchy_errors > 0 %then 
   %do;
      %let retcode=2;
   %end;    

 %dq_provhier_end:
    %if &retcode=0 %then %do;
       proc sql;
          drop table &outdsn;
       quit; 
    %end;
%mend;
