/*******************************************************************************
 *  $Revision: 1.1.4.19.10.1 $
 *
 *  $Date: 2017/09/07 19:25:44 $
 *
 *  Macro:       sea_load_rpt_tables
 *
 *  Author:      Chris Diering
 *
 *  Date:        October, 2014
 *
 *  Purpose:     Load VA report data into Lasr
 * 
 *  Parameters:  None
 *
 *  Macro Calls: avacmn_lasr_loader
 *               sea_utl_chk_rpt_logs
 *
 *
 *  History      | ID      | DATE        | COMMENT
 *               | chdier  | 11/04/2014  | Initial creation
 *               | cabahl  | 05/27/2015  | 3.1 update
 *               | cabahl  | 08/20/2015  | Removed user playpen creation
 *               | cabahl  | 09/05/2017  | 3.12 Hotfix HAF_AUDIT update    
 *
 *  Notes:
 *
 *******************************************************************************/

/*==================================================================================
Copyright (c) 2014 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
***********************************************************************************/

%macro sea_load_rpt_tables(rptmart=
                          ,repository_guid= 
                          );

    /*-----------------------------------------------------------------------------------
    Initialize the retcode to 0.  Get just the errors from the temp.sea_rpt_msgs.  If there
    are any errors the retcode macro variable will be set to 1. If retcode is 1 then
    issue Error and error out.
    -----------------------------------------------------------------------------------*/
    %global retcode;
    %let retcode = 0;

    /*-----------------------------------------------------------------------------------
    Get LASR Library OMR path
    -----------------------------------------------------------------------------------*/
    data _null_;
      set sashelp.vslib (where=(libname = 'RPTMART'));
      length uri uri_t uri_t1 uri_o lasr_lib_nm folder_nm $256. 
             folder_path directory $1000 engine libref $15;
      n=1;
      nobj=0;
      query1="omsobj:SASLibrary?*[Extensions/Extension[@Value='%ktrim(%kleft(%bquote(&REPOSITORY_GUID)))']]";
      
      do while(nobj ge 0);
        uri='';
        nobj=metadata_getnobj(query1,n,uri);
        if nobj > 0 then do;
          rc=metadata_getattr(uri,"Engine",engine);
          if engine='SASIOLA' then do;
            /* Get LASR library information */
            rc=metadata_getattr(uri,"Name",lasr_lib_nm);
            rc=metadata_getattr(uri,'Libref',libref);
            rc=metadata_getnasn(uri,"Trees",1,uri_t);
            rc=metadata_getattr(uri_t,"Name",folder_nm);
            
            rc=metadata_getnasn(uri,"Trees",1,uri_t);
            rc=metadata_getattr(uri_t,"Name",folder_nm);     

            folder_path=folder_nm;

            rc=metadata_getnasn(uri_t,"ParentTree",1,uri_t1);
            rc=metadata_getattr(uri_t1,"Name",folder_nm);

            put lasr_lib_nm= folder_nm= folder_path= ;
            
            tobj=1;
            do while(tobj > 0);
              tobj=metadata_getnasn(uri_t,"ParentTree",1,uri_t1);
              rc=metadata_getattr(uri_t1,"Name",folder_nm);
              if tobj > 0 then do;      
                folder_path=strip(folder_nm)||'/'||strip(folder_path);
              end;
              uri_t=uri_t1;
              uri_t1='';
              folder_nm='';
            end;

            call symput('sea_rpt_lasr_folder','/'||strip(folder_path));
            call symput('sea_rpt_lasr_library',strip(lasr_lib_nm));
            call symput('lasr_libref',libref);
            output;
          end;
          else if engine="BASE" then do;
            /* RPTMART library - get directory */
            rc=metadata_getnasn(uri,"UsingPackages",1,uri_o);
            rc=metadata_getattr(uri_o,"DirectoryName",directory);
            /* if directory does not match path then update with path */
            if strip(directory) ne strip(path) then rcu=metadata_setattr(uri_o,"DirectoryName",path);
            output;
          end;
          n=n+1;
        end;
      end;  
    run;
    
    %put folder_path = &sea_rpt_lasr_folder;
    %put lasr_lib_nm = &sea_rpt_lasr_library;

    %if &sea_rpt_lasr_library = %str() %then %do;
       %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds ,_sea_loadrpt01_error,noquote,&repository_nm));
       %return;
    %end;

    /*check for report output table existence */
    proc sql;
        create view work.rptmart_tables as 
          select *
            from dictionary.tables
           where upcase(libname) =upcase("&rptmart");
    quit;

    /*-----------------------------------------------------------------------------------
    Go through the output tables and create a macro var for each table name.  Also create
    a macro variable called number_of_report_tables that contain the number of tables.
     -----------------------------------------------------------------------------------*/
    %let number_of_report_tables=0;
    data _null_;
        set work.rptmart_tables;
        call symputx('number_of_report_tables',ktrim(kleft(put(_n_,best12.))),G);
        call symputx('report_table_name'||kleft(ktrim(put(_n_,best12.))),memname,G);
    run;

    %if &number_of_report_tables=0 %then %do;
      %return;
    %end;

    /*----------------------------------------------------------------------------------
    * Delete all tables within the lasr library
    *---------------------------------------------------------------------------------*/
    %avacmn_lasr_loader(action=CLEANOUT
                       ,folder_lasr=&sea_rpt_lasr_folder
                       ,lasr_lib=&sea_rpt_lasr_library 
                       );
    %let libcnt=0;
    proc sql noprint;
      select count(*)
        into :libcnt
        from dictionary.tables
        where libname=upcase("&lasr_libref");
    quit;

    %if &libcnt > 0 %then %do;
       %let retcode=1;
       %return;
    %end;

    /*----------------------------------------------------------------------------------
    *   Load tables into VA 
    *---------------------------------------------------------------------------------*/
    %do i=1 %to &number_of_report_tables;
        %put ***************START REPLACE_TABLE &&report_table_name&i ************************;

        /*-----------------------------------------------------------------------------------
           Create the load_ds_desc macro var by combining the table name with the load datetime.
           This will be used for the description of the loaded lasr table.
        -----------------------------------------------------------------------------------*/
        data _null_;
            length load_ds_desc $100.;
            load_ds_desc = "&&report_table_name&i"||' Loaded '||ktrim(kleft(put(datetime(),datetime20.)));
            call symputx('load_ds_desc',load_ds_desc,'G');
            put load_ds_desc=;
        run;

        /*-----------------------------------------------------------------------------------
           Use the avacmn_lasr_loader macro to load the table into lasr.
        -----------------------------------------------------------------------------------*/
        %let action=REPLACE_TABLE;
        %avacmn_lasr_loader(lasr_table=&&report_table_name&i  
                           ,source_dsn=&&report_table_name&i
                           ,source_lib=&rptmart    
                           ,action=&action
                           ,folder_lasr=&sea_rpt_lasr_folder
                           ,lasr_lib=&sea_rpt_lasr_library 
                           ,row_del_criteria=
                           ,partition=
                           ,orderby=
                           ,ds_desc=&load_ds_desc  
                         %if &i < &number_of_report_tables %then %do;
                           ,metadata_update=N
                         %end;
                         %else %do;
                           ,metadata_update=ALL
                         %end; 
                           );
                             
        %if %sysfunc(exist(&lasr_libref..&&report_table_name&i)) = 0 %then %do;
           %let retcode=1;
           %let action=DELETE_TABLE;
        %end;
        
        data audit(drop=action i);
         length
          user_id           $100
          event_datetime    8
          workspace_id      $10
          workspace_nm      $100
          cohort_id         $10
          cohort_nm         $100
          owner_id          $100
          job_id            $10
          job_nm            $100
          file_set_id       $36
          file_set_nm       $60
          addin_id          $36
          addin_nm          $100
          group_id          $10
          group_nm          $100
          operation         $100
          component_id      $36
          component_nm      $40
          ;
          array vars{*} $ user_id workspace_id workspace_nm cohort_id
                          cohort_nm owner_id job_id job_nm file_set_id
                          file_set_nm addin_id addin_nm group_id group_nm
                          operation;
  
          do i=1 to dim(vars);
            vars{i}='';
          end;
  
          %if %symexist(USERID) %then %do;
            user_id="&userid";
          %end;
  
          %if %symexist(REPOSITORY_NM) %then %do;
            workspace_nm="&repository_NM";
          %end;
          
          %if %symexist(REPOSITORY_GUID) %then %do;
            workspace_id="&repository_GUID";
          %end;    

          component_id="&i";
          component_nm="&&report_table_name&i"; 
    
          event_datetime=datetime();
  
          action="&action";
  
          /* Actions
             REPLACE_TABLE = replace complete table or a specific set of rows
             DELETE_TABLE  = deletes the table within the LASR library and
                             within the OMR
          */
          select(action);
            when('REPLACE_TABLE') operation="LASR_TableUpdate";
            when('DELETE_TABLE') operation="LASR_TableDelete";
            otherwise;
          end;
  
        run;
  
        proc append base=&oralib..HAF_AUDIT
                    data=audit;
        run;
  
        %put &sysrc;        
                
    %end;

   /* Clean up */
   libname &lasr_libref clear;

%mend sea_load_rpt_tables;
