/*******************************************************************************
 *  $Revision: 1.1.4.11.10.1 $
 *  $Date: 2017/09/07 19:25:44 $
 *
 *  Macro:       sea_va_create_bndld_pymnt_bdgt
 *
 *  Author:      Caroline Bahler
 *
 *  Date:        October, 2014
 *
 *  Purpose:     Create to create risk and budget data for bundled payment
 *               reporting
 * 
 *  Parameters: 
 *
 *
 *  History      | ID      | DATE        | COMMENT
 *               | cabahl  | 10/13/2014  | Initial creation
 *               | cabahl  | 11/18/2014  | DE5012 added parameter checks
 *               | cabahl  | 12/02/2014  | DE5065 added fields to SEA_RA_PREDCOST
 *                                         filter
 *               | cabahl  | 12/08/2015  | S1221963 add setting missing values to 0
 *               | cabahl  | 09/05/2017  | S1368642 remove episode_coverage filter from 
 *                                         risk data query 
 *  Notes:
 *
 *******************************************************************************/

/*==================================================================================
Copyright (c) 2014 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
***********************************************************************************/

%macro sea_va_create_bndld_pymnt_bdgt(risk_data=ra.sea_ra_predcost
                                     ,episodes_data=mart.sea_mart_episodes
                                     ,conditions_data=mart.sea_mart_conditions
                                     ,services=
                                     ,vamart=
                                     );

  %global retcode;
  %let retcode=0;

  %if %nrbquote(&risk_data) eq %then
      %do;
          %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_parm_missing_error,noquote,RISK_DATA)));
      %end;

  %if %sysfunc(exist(&risk_data, DATA)) = 0 and %sysfunc(exist(&risk_data, VIEW)) = 0 %then
      %do;
        %let msg=%sysfunc(sasmsg(&sea_message_ds,_sea_rp_001_note,noquote,));
        %put %sysfunc(sasmsg(&sea_message_ds,_log_note,noquote)) &msg;
        %return;
      %end;

  %if %nrbquote(&episodes_data) eq %then
      %do;
          %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_parm_missing_error,noquote,EPISODES_DATA)));
      %end;

  %if %sysfunc(exist(&episodes_data, DATA)) = 0 and %sysfunc(exist(&episodes_data, VIEW)) = 0 %then
      %do;
          %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_dataset_notexist_error,noquote,&episodes_data)));
      %end;
      
  %if %nrbquote(&services) eq %then
      %do;
          %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_parm_missing_error,noquote,SERVICES)));
      %end;

  %if %sysfunc(exist(&services, DATA)) = 0 and %sysfunc(exist(&services, VIEW)) = 0 %then
      %do;
          %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_dataset_notexist_error,noquote,&services)));
      %end; 
      
  %if %nrbquote(&vamart) eq %then
      %do;
          %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_parm_missing_error,noquote,VAMART)));
      %end;
      
  %if &retcode=1 %then %do;
    %return;
  %end;

   /* Condition description, type */
    data cndtn;
        set &conditions_data(keep=cond:  
            rename=(condition_cd=start))
            end=eof;
        hlo='';
        fmtname='$condition';
        label=compbl(condition_desc);
        output;

        if eof then
            do;
                hlo='O';
                start='OTHER';
                label='';
                output;
            end;
    run;

    proc format cntlin=cndtn;
    run;

  /* 
      Create data for Bundled Payment Report B
      - Provider Variance
      - Episode Budget
      - Provider Budget
  */
  /* Get episode start and end dates 
      - using only those episodes that meet all filter requirements */
  proc sort data=&episodes_data
             out=episodes(keep=epi_id epi_start_dt epi_end_dt);
    where epi_fltr_all_limits_flg='Y';
    by epi_id;
  run;

  /* Get output of risk analysis
     Pull data that has a provider attributed and has complete episode information
  */
  data risk_data;
    set &risk_data;
    WHERE model_period = 'Entire Episode' and
          episode_cost_leveling='CL' 
          /* S1368642 and episode_coverage='Entire Episode'*/;
    /* Set expected and actual amounts to 0 if missing S1221963 */
    if exp_typical_amt=. then exp_typical_amt=0;
    if exp_pac_amt=. then exp_pac_amt=0;
    if comp=. then comp=0;
    if typical=. then typical=0;
  run;

  PROC SQL;
     CREATE TABLE risk AS 
     SELECT t1.condition, 
          t1.epi_id, 
          t1.exp_typical_amt/t2.no_providers as exp_typical_amt, 
          t1.exp_pac_amt/t2.no_providers as exp_pac_amt, 
          t1.Typical/t2.no_providers as Typical, 
          t1.Comp/t2.no_providers as Comp, 
          t2.provider_id     
     FROM  risk_data t1
           inner JOIN RA.RA_PROV_ATTRS t2 ON (t1.epi_id = t2.epi_id)
     order by t1.epi_id;
  QUIT;


  data &vamart..sea_va_bndld_payment_risk_adj;
    merge risk(in=a rename=(comp=pac_amt typical=typical_amt
                            condition=condition_cd       
                           ) 
               )
          episodes(in=b);
    by epi_id;
    label provider_id='Attributed Provider'
          typical_amt='Typical Cost' 
          pac_amt ='PAC Cost'
          exp_typical_amt ='Expected Typical Cost'
          exp_pac_amt='Expected PAC Cost'
          total_cost ='Episode Total Cost'
          exp_total_cost='Total Expected Cost' 
          condition='Condition' 
          year='Year'
    ;
    length condition $40;
    if a then do;
      total_cost=typical_amt+pac_amt;
      exp_total_cost=exp_typical_amt+exp_pac_amt;
    end;
    condition=put(condition_cd,$condition.);
    year=year(epi_end_dt);
    if a and b then output;
  run;
 

  /* Claim Detail */
  data &vamart..sea_va_bndld_payment_risk_dtl(drop=pr rcd);
    merge  &vamart..sea_va_bndld_payment_risk_adj(keep=epi_: provider_id condition: in=a)
           &SERVICES(keep=epi_id claim: srvc_lvl: srvc_assign: srvc_reason: srvc_allocated_amt
                          srvc_period_in_epi_cd epi_fltr_all_limits_flg service_:
                     in=b);
    by epi_id;
    length pac_reason service_group $250;
    /* Labels */
    label service_group='Services' 
          pac_reason='PAC Reason'
          c_allocated_amt='PAC Costs'
          tc_allocated_amt='Typical Costs'
          t_allocated_amt='Typical with Comp Costs'
          claim_id='Claim Id'
          claim_line_no='Line Id'
    ;
    if a and b then do;
      c_allocated_amt=0;
      t_allocated_amt=0;
      tc_allocated_amt=0; 

      /* Service group */
      rcd=compress(upcase(srvc_reason_px_vocab_cd||'PX'||
                          srvc_reason_px_code_val));
      service_group=strip(put(compress(rcd),$hcigrp.));

      if claim_type_cd='RX' then do;
         service_group='RX';
      end;

      /*
      * creating allocated amt variables based on assignment type
      */
      select(srvc_assignment_type_cd);
        when('C') do;
           c_allocated_amt=srvc_allocated_amt; 
           pr=compress(upcase(srvc_reason_dx_vocab_cd||'DX'||
                              srvc_reason_dx_code_val));
           pac_reason=strip(put(compress(pr),$hcigrp.));
        end;
        when('T') do;
           t_allocated_amt=srvc_allocated_amt; 
        end;
        when('TC') do;
           tc_allocated_amt=srvc_allocated_amt; 
        end;
        otherwise;
      end;
      ;
      output;
    end;
  run;

  /* Check for report data field name case and correct if necessary
     required due to VA reports require specific case of field names */
    data _null_;
      set sashelp.vcolumn end=eof;
      where libname=upcase("&vamart")
        and memname in ("SEA_VA_BNDLD_PAYMENT_RISK_DTL");
      retain c 0;
      if lowcase(name) ne name then do;
         c=c+1;
         call symput(compress('t'||put(c,best.)),memname);
         call symput(compress('rn'||put(c,best.)),compress(name||'='||lowcase(name)));
      end;

      if eof then call symput('cc',compress(put(c,best.)));
    run;

    %if &cc > 0 %then %do;
       proc datasets lib=&vamart nolist;
         %do i=1 %to &cc;
            modify &&t&i;
            rename &&rn&i;
            run;
         %end;
       quit;
    %end;
%mend;
